#!/usr/bin/perl
#
#  slash2spanning.pl: Convert multiple C++ "slash-slash" style comment lines
#                     into one spanning C-style comment (/* ... */), ignoring
#                     lines that start with "slash-slash-hash style comments.
#
#  $Id: slash2spanning.pl,v 1.8 2005/01/27 14:44:18 loose Exp $

$NON_COMMENT_BLOCK = 0;
$COMMENT_BLOCK     = 1;

# Defines fileparse()
use File::Basename;

# Only files whose suffix match with that in the suffixlist will be filtered;
# other files will be printed unmodified.
@suffixlist = ('\.h', '\.idl');

# If the caller supplied precisely one command line argument, then this script
# can proceed.
$arguments = @ARGV;
if ($arguments == 1)
{
  # The argument represents the name of the file to process.
  $filename = $ARGV[0];

  # Check if we need to filter the input file.
  ($name, $path, $suffix) = fileparse($filename, @suffixlist);
  $doFilter = $suffix ne "";

  # Open the file.
  $status = open(FILEHANDLE, $filename);

  # If the file could be opened then read the contents.
  if ($status)
  {
    # Read the contents of the file.
    @text = <FILEHANDLE>;

    # Close the file.
    close(FILEHANDLE);

    # Process the contents of the file.
    if ($doFilter)
    {
      change_comment_style(@text);
    }
    else
    {
      print @text
    }
  }

  # Otherwise report an error condition.
  else
  {
    print "ERROR: file \"$filename\" could not be opened.\n";
  }
}

# Otherwise, display the proper invocation for this script.
else
{
  print "usage: $0 filename\n";
}

sub change_comment_style
{
  $keephash = 1;
  $state = $NON_COMMENT_BLOCK;
  $newstate = $NON_COMMENT_BLOCK;
  @spanning_comment = ();
  $indent = "";
  $comment = "";

  foreach $line (@_)
  {
    $newstate = $NON_COMMENT_BLOCK;
    # slash-slash-hash is blanked if not at beginning of file..
    if ($keephash == 0  &&  $line =~ /^\s*\/\/\#/)
    {
      $line = "\n";
    }
    $comment = $line;
    # If this line contains only a slash-slash comment, then
    # the comment block might need to be converted.
    if ($line =~ /(^\s*)(\/\/)(.*)/)
    {
      # Keep track of how far the comment block is indented.
      $indent = $1;
      $comment = $3;
      # slash-slash-slash/hash are kept as such (is doxygen already).
      if ($comment !~ /^[\/#]/)
      {
        $keephash = 0;
	$newstate = $COMMENT_BLOCK;
	# Replace <srcblock> by \code
	$comment =~ s/<srcblock>/\\code/g;
	$comment =~ s/<\/srcblock>/\\endcode/g;
	# Replace <summary> by \brief
	$comment =~ s/<summary>/\\brief/g;
	$comment =~ s/<\/summary>//g;
	# Handle group comments and keep // in doxygen groups.
	$comment =~ s/<group>/@\{/g;
	$comment =~ s/<\/group>/@\}/g;
	if ($comment =~ /^\s*@[\{\}]\s*$/)
	{
	  $comment =~ s/\s*//g;
	  $line = "$indent//$comment\n";
	  $newstate = $NON_COMMENT_BLOCK;
	}
      }
      # A blank line does not change anything.
      elsif ($line =~ /^\s*$/)
      {
	$newstate = $state;
      }
    } else {
      $keephash = 0;
    }

    # Act depending on new and old state.
    if ($newstate == $COMMENT_BLOCK) {
      if ($state == $COMMENT_BLOCK) {
	# Add comment to block.
	push @spanning_comment, "\n$indent   $comment";
      } else {
	# Begin the spanning comment block with given indentation.
	push @spanning_comment, "$indent\/**$comment";
      }
    } else {
      if ($state == $COMMENT_BLOCK) {
	# End comment block and write the block.
	push @spanning_comment, " */\n";
	print @spanning_comment;
	@spanning_comment = ();
      }
      # Write the line.
      print $line;
    }
    $state = $newstate;
  }

  # Output remaining comments if there.
  if ($state == $COMMENT_BLOCK)
  {
    push @spanning_comment, " */\n";
    print @spanning_comment;
  }
}

